@@ -195,7 +195,7 @@ def GPGME.verify(sig, *args_options) # :yields: signature
   ctx.verify_result.signatures.each do |signature|
     yield signature
   end
-  unless plain
+  if !signed_text && !plain
     plain_data.seek(0, IO::SEEK_SET)
     plain_data.read
   end
@@ -236,7 +236,7 @@ def GPGME.sign(plain, *args_options)
   sig = args[0]
 
   ctx = GPGME::Ctx.new(options)
-  ctx.add_signer(resolve_keys(options[:signers]), true) if options[:signers]
+  ctx.add_signer(*resolve_keys(options[:signers], true)) if options[:signers]
   mode = options[:mode] || GPGME::SIG_MODE_NORMAL
   plain_data = input_data(plain)
   sig_data = output_data(sig)
@@ -360,7 +360,7 @@ def GPGME.encrypt(recipients, plain, *args_options)
   begin
     if options[:sign]
       if options[:signers]
-        ctx.add_signer(resolve_keys(options[:signers]), true)
+        ctx.add_signer(*resolve_keys(options[:signers], true))
       end
       ctx.encrypt_sign(recipient_keys, plain_data, cipher_data)
     else
@@ -501,7 +501,7 @@ module GPGME
       if key_or_name.kind_of? Key
         keys << key_or_name
       elsif key_or_name.kind_of? String
-        keys += ctx.keys(key_or_name)
+        keys += ctx.keys(key_or_name, secret_only)
       end
     end
     keys
@@ -1066,7 +1066,7 @@ keylist_mode=#{KEYLIST_MODE_NAMES[keylist_mode]}>"
 
     # Extract the public keys of the recipients.
     def export_keys(recipients, keydata = Data.new)
-      err = GPGME::gpgme_op_export(self, recipients, keydata)
+      err = GPGME::gpgme_op_export(self, recipients, 0, keydata)
       exc = GPGME::error_to_exception(err)
       raise exc if exc
       keydata
@@ -1172,7 +1172,7 @@ keylist_mode=#{KEYLIST_MODE_NAMES[keylist_mode]}>"
     end
 
     def encrypt_result
-      GPGME::gpgme_encrypt_result(self)
+      GPGME::gpgme_op_encrypt_result(self)
     end
 
     def encrypt_sign(recp, plain, cipher = Data.new, flags = 0)
